"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ensureDirectory = exports.makeReadSourceFileAstProvider = exports.RtoProjectOutput = void 0;
const fs = require("fs");
const path_1 = require("path");
const util_1 = require("util");
const module_path_helpers_1 = require("../helpers/module-path-helpers");
const parse_typeonly_1 = require("../parser/parse-typeonly");
const readFile = (0, util_1.promisify)(fs.readFile);
const writeFile = (0, util_1.promisify)(fs.writeFile);
const mkdir = (0, util_1.promisify)(fs.mkdir);
class RtoProjectOutput {
    constructor(options) {
        this.modules = new Map();
        this.listener = async (module, modulePath) => {
            if (options.writeFiles)
                await this.writeModuleFile(module, modulePath, options.writeFiles);
            if (options.returnRtoModules)
                this.modules.set(modulePath, module);
        };
    }
    getRtoModules() {
        const result = {};
        for (const [modulePath, module] of this.modules)
            result[modulePath] = module;
        return result;
    }
    async writeModuleFile(module, modulePath, options) {
        const data = JSON.stringify(module, undefined, options.prettify);
        const outputFile = `${(0, path_1.join)(options.outputDir, modulePath)}.rto.json`;
        await ensureDirectory(options.outputDir);
        await ensureDirectory((0, path_1.dirname)(outputFile), { createIntermediate: true });
        await writeFile(outputFile, data, { encoding: options.encoding });
    }
}
exports.RtoProjectOutput = RtoProjectOutput;
function makeReadSourceFileAstProvider(sourceDir, encoding) {
    return async (modulePath) => {
        const source = await readModuleFile(sourceDir, modulePath, encoding);
        return (0, parse_typeonly_1.parseTypeOnlyToAst)(source);
    };
}
exports.makeReadSourceFileAstProvider = makeReadSourceFileAstProvider;
async function readModuleFile(sourceDir, modulePath, encoding) {
    const parsedExternalModule = (0, module_path_helpers_1.parseExternalModuleName)(modulePath);
    const path = parsedExternalModule
        ? await (0, module_path_helpers_1.getExternalModulePath)(parsedExternalModule, sourceDir)
        : (0, path_1.join)(sourceDir, modulePath);
    try {
        return await readFile(`${path}.d.ts`, encoding);
    }
    catch (_a) {
    }
    try {
        return await readFile(`${path}.ts`, encoding);
    }
    catch (_b) {
    }
    throw new Error(`Cannot open module file: ${path}.d.ts`);
}
async function ensureDirectory(path, { createIntermediate = false } = {}) {
    if (!await fsExists(path))
        await mkdir(path, { recursive: createIntermediate });
}
exports.ensureDirectory = ensureDirectory;
function fsExists(path) {
    return new Promise(resolve => {
        fs.stat(path, error => resolve(!error));
    });
}
